#!/usr/bin/env bash

. /mnt/mod/ctrl/configs/functions &>/dev/null 2>&1

ACTION=$(basename $0 | cut -d- -f 2 | cut -d. -f 1)
if [ "$ACTION" != "enable" -a "$ACTION" != "disable" ]; then
  echo "Unexpected action: ${ACTION}"
  exit 1
fi

# Where are we? (Could be TF1 or TF2)
HERE=$(cd $(dirname $0); pwd)

# Set up some directories for our use.
DATA_DIR="${HERE}/ssh"

BACKUP_DIR="${DATA_DIR}/backup"
CONFIG_DIR="${DATA_DIR}/config"
LOG_DIR="${DATA_DIR}/log"
USERS_DIR="${DATA_DIR}/users"
if ! mkdir -p "$DATA_DIR" "$BACKUP_DIR" "$CONFIG_DIR" "$LOG_DIR" "$USERS_DIR"; then
  echo "Unable to create required directories for $(basename $0)"
  exit 1
fi


# Setup some file paths.
TIME=$(date '+%s')

# Log file to hold all output of this script.
ACTION_LOG_FILE="${LOG_DIR}/${TIME}-${ACTION}.log"
# Log file to hold output of the running service.
SERVICE_LOG_FILE="${LOG_DIR}/${TIME}-service.log"
# Backup file and original for overwriting sshd_config.
ORIGINAL_SSHD_CONFIG="/etc/ssh/sshd_config"
NEW_SSHD_CONFIG="${CONFIG_DIR}/sshd_config"
BACKUP_SSHD_CONFIG="${BACKUP_DIR}/sshd_config_${TIME}"
# Filenames representing statuses. Used to rename this file after changing state.
NEXT_SCRIPT_NAME_DISABLE="${HERE}/ssh-disable.sh"
NEXT_SCRIPT_NAME_ENABLE="${HERE}/ssh-enable.sh"
# A relative path to place our managed files passed to sshd_config.
USER_RELATIVE_SSH_DIR="ssh-enable-conf"

# Remap outputs to our log file.
exec 3>&1 4>&2
exec >"$ACTION_LOG_FILE" 2>&1

handler() {
  echo "Error running '${ACTION}'.$(echo)Check ${ACTION_LOG_FILE}."
  # Restore our remapped outputs.
  exec 1>&3 2>&4
}
trap handler ERR SIGINT

# Add some error handling.
set -e
set -x

if [ "$ACTION" == "enable" ]; then
  echo "Enabling ssh service..."

  for USER_DIR in $(find "$USERS_DIR" -maxdepth 1 -type d); do
    echo "Preparing user directory: ${USER_DIR}"
    if [ -d "$USER_DIR" ]; then
      USER=$(basename "$USER_DIR")
      echo "Found user: $USER"
      if id "$USER"; then
        # Make sure our user directories have the proper permissions.
      	if [ -f "$USER_DIR/authorized_keys" ]; then
      	  USER_HOME=$(getent passwd "$USER" | cut -d: -f6)
      	  if [ ! -d "$USER_HOME" ]; then
        	  mkdir -p "$USER_HOME"
      	    chown $USER:$USER "$USER_HOME"
          fi
          chmod 755 "$USER_HOME"
      	  USER_SSH_DIR="${USER_HOME}/${USER_RELATIVE_SSH_DIR}"
      	  if [ ! -d "$USER_SSH_DIR" ]; then
      	    mkdir -p "$USER_SSH_DIR"
      	    chown $USER:$USER "$USER_SSH_DIR"
          fi
          chmod 700 "$USER_SSH_DIR"
      	  cp "$USER_DIR/authorized_keys" "${USER_SSH_DIR}/authorized_keys"
          chown $USER:$USER "${USER_SSH_DIR}/authorized_keys"
          chmod 644 "${USER_SSH_DIR}/authorized_keys"
        fi
      fi
    fi
  done

  # Make a backup and install our modified sshd_config
  if [ -f "$ORIGINAL_SSHD_CONFIG" ]; then
    mv "$ORIGINAL_SSHD_CONFIG" "$BACKUP_SSHD_CONFIG"
  fi
  if [ -f "$NEW_SSHD_CONFIG" ]; then
    cat "$NEW_SSHD_CONFIG" | sed "s#@@AUTHORIZED_KEYS_FILE_BASE@@#${USER_RELATIVE_SSH_DIR}#" | tee "$ORIGINAL_SSHD_CONFIG"
    chmod 644 "$ORIGINAL_SSHD_CONFIG"
  fi

  # Actually enable the thing.
  systemctl enable ssh
  systemctl start ssh >"$SERVICE_LOG_FILE" &

  # Rename script to reflect status in menus.
  mv "$0" "$NEXT_SCRIPT_NAME_DISABLE" && sync
fi

if [ "$ACTION" == "disable" ]; then
  echo "Disabling ssh service..."

  # Actually disable this thing.
  systemctl disable ssh

  # Rename script to reflect status in menus.
  mv "$0" "$NEXT_SCRIPT_NAME_ENABLE" && sync
fi

